from itchat.storage.templates import Chatroom as ITChatroom
from .interface import BaseModel

class User(BaseModel):
    def __init__(self,nickname:str, username:str) -> None:
        self.nickname = nickname
        self.username = username
    
    def get_username(self):
        return self.username 
    
    def get_nickname(self):
        return self.nickname
    def __str__(self) -> str:
        return str(self.to_dict())
    
    def __repr__(self) -> str:
        return self.__str__()
    

class Chatroom(User):
    @classmethod
    def build_from_ITChatroom(cls, object:ITChatroom):
        return Chatroom(object['NickName'], object['UserName'])


import itchat
from .user import Chatroom
from .userlist import UserList
from threading import Thread
import time

class ICHelper:
    
    def check_login(self, timer=5):
        def work():
            while True:
                print(itchat.check_login())
                time.sleep(timer)
        Thread(target=work).start()
    
    def login(self, hot_reload=False, picDir='qrcode.png', new_thread=False,login_callback=None):
        def _login(hot_reload, picDir):
           itchat.auto_login(hotReload=hot_reload, picDir=picDir,loginCallback=login_callback)  
           
        if new_thread:
            t = Thread(target=_login, args=(hot_reload, picDir))
            t.start()
            return t
        else:
            _login(hot_reload, picDir)
    
    

    def get_chatrooms(self):
        # 获取所有群组
        chatrooms = itchat.get_chatrooms()
        return UserList([Chatroom.build_from_ITChatroom(x) for x in chatrooms])
    
    def send_chatroom_msg(self,msg:str, chatroom_nickname:str):
        chatrooms = self.get_chatrooms()
        room = chatrooms.get_user_by_nickname(chatroom_nickname)
        if room:
            itchat.send_msg(msg, toUserName=room.get_username())
            return True
        else:
            return False
    
    def logout(self):
        itchat.logout()
            
    
    
        
    


from .user import User


class UserList:
    def __init__(self, users: list[User]) -> None:
        self.__list__ = users

    def get_user_by_nickname(self, nickname: str):
        for u in self.__list__:
            if u.nickname == nickname:
                return u
        return None
    
    def __str__(self) -> str:
        return str(self.__list__)
    
    def __repr__(self) -> str:
        return str(self.__list__)
    
    # subscriptable and slice-able
    def __getitem__(self, idx):
        return self.__list__[idx]
    
     # return an iterator that can be used in for loop etc.
    def __iter__(self):
        return self.__list__.__iter__()

    def __len__(self):
        return len(self.__list__)



from setuptools import setup, find_packages

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

requirements = ["itchat",] 

setup(
    name="itchat-helper",
    version="0.1.0",
    author="Tao Xiang",
    author_email="tao.xiang@tum.de",
    description="",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="",
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
  "Programming Language :: Python :: 3",
  "License :: OSI Approved :: MIT License",
    ],
)


import json

class IO:
    @classmethod
    def dict_to_json(cls, dictObj: dict, file_path:str):
        jsObj = json.dumps(dictObj)
        with open(file_path, 'w') as f:
            f.write(jsObj)


    @classmethod
    def json_to_dict(cls, file_path:str) -> dict:
        with open(file_path, 'r', encoding='UTF-8') as f:
            load_dict = json.load(f)
        return load_dict
       
       
class BaseModel(IO):
    def __init__(self,**kwargs):
        for key,value in kwargs.items():
            self.__setattr__(key,value)

    def to_dict(self) -> dict:
        ret = {}
        for key,value in self.__dict__.items():
            if hasattr(value,'to_dict'):
                ret[key] = value.to_dict()
            else:
                ret[key] = value
        return ret

    def to_json(self, path:str):
        self.dict_to_json(self.to_dict(),path)

    @classmethod
    def from_dict(cls,dictobj: dict):
        return cls(**dictobj)


    @classmethod
    def from_json(cls,path:str):
        return cls.from_dict(cls.json_to_dict(path))


    def __eq__(self, other):
        if not isinstance(other,self.__class__):
            return False
        else:
            for k,v in other.__dict__.items():
                if not hasattr(self,k):
                    return False
                else:
                    if self.__getattribute__(k) != v:
                        return False

            return True


    def __copy__(self):
        return self.__class__(**self.to_dict())
    
    def __deepcopy__(self, memodict={}):
        return self.__copy__()